package com.sw.appcloudv1.Utils;

import org.apache.commons.codec.digest.DigestUtils;

/**
 * 短连接算法
 * Created by shiwn on 2022/2/24 10:07
 */
public class ShortUrlUtils {
    //  26小写字母 + 26大写字母 + 10个数字 = 62
    public static final String[] chars = new String[]{"a", "b", "c", "d", "e", "f", "g", "h",
            "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t",
            "u", "v", "w", "x", "y", "z", "0", "1", "2", "3", "4", "5",
            "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H",
            "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T",
            "U", "V", "W", "X", "Y", "Z"};

    /**
     * 短连接算法
     */
    public static String[] shortUrl(String url) {
        //  对传入网址进行 MD5 加密
        String sMD5EncryptResult = DigestUtils.md5Hex(url);
        //  md5处理后是32位
        String hex = sMD5EncryptResult;
        //  切割为4组，每组8个字符, 32 = 4 *  8
        String[] resUrl = new String[4];

        for (int i = 0; i < 4; i++) {
            //  取出8位字符串，md5 32位，按照8位一组字符,被切割为4组
            String sTempSubString = hex.substring(i * 8, i * 8 + 8);

            //  把加密字符按照8位一组16进制与 0x3FFFFFFF 进行位与运算
            //  这里需要使用 long 型来转换，因为 Inteper .parseInt() 只能处理 31 位 , 首位为符号位 , 如果不用 long ，则会越界
            long lHexLong = 0x3FFFFFFF & Long.parseLong(sTempSubString, 16);

            String outChars = "";
            for (int j = 0; j < 6; j++) {
                //  0x0000003D它的10进制是61，61代表最上面定义的chars数组长度62的0到61的坐标。
                //  0x0000003D & lHexLong进行位与运算，就是格式化为6位，即保证了index绝对是61以内的值
                long index = 0x0000003D & lHexLong;
                //  按照下标index把从chars数组取得的字符逐个相加
                outChars += chars[(int) index];
                //  每次循环按位移5位，因为30位的二进制，分6次循环，即每次右移5位
                lHexLong = lHexLong >> 5;
            }
            // 把字符串存入对应索引的输出数组,会产生一组6位字符串
            resUrl[i] = outChars;
        }
        return resUrl;
    }
}
